/*
Chronomtre officiel pour jeu de ptanque
Copyleft Fabrice Sincre
///////////////////////////////////////////
// Licence : GNU General Public License  //
///////////////////////////////////////////
Lyce Jules Algoud (Valence, France)
Version 1.0.11.06
Cible : Microcontrleur PIC 16F628A
Langage C : compilateur mikroC v8.2

Bits de configuration : 0x3F22
_BOREN_OFF = $3FBF
_CP_OFF = $3FFF
_DATA_CP_OFF = $3FFF
_PWRTE_ON = $3FF7
_WDT_OFF = $3FFB
_LVP_OFF = $3F7F
_MCLRE_ON = $3FFF
_HS_OSC = $3FEE   (quartz 10 ou 20 Mhz, cf. OPTION_REG plus bas)

*/

# define COMPTEUR_INIT       257
# define COMPTEUR_TMR0_INIT  38

# define COMPTEUR_BP_INIT         40

// Port A
# define BIT_ANODE_DIGIT_DIZAINE   F3
# define BIT_ANODE_DIGIT_UNITE     F4

// Port B
# define BIT_BP_REPRISE            F0
# define BIT_BP_RESET              F1
# define BIT_7447_A                F2
# define BIT_7447_B                F3
# define BIT_7447_C                F4
# define BIT_7447_D                F5
# define BIT_BUZZER                F6
# define BIT_BP_START_PAUSE        F7

// dclaration des fonctions

void rafraichissement();
unsigned short int test_bp_reset();
unsigned short int test_bp_reprise();
unsigned short int test_bp_start_pause();

// dclaration des variables globales

unsigned int compteur = COMPTEUR_INIT;           // entier non sign 2 bytes (0  65535)
unsigned short int compteur_tmr0 = COMPTEUR_TMR0_INIT; // entier non sign 1 byte (0  255)
unsigned short int compteur_bp = COMPTEUR_BP_INIT;

unsigned short int affichage_digit_dizaine = 6;
unsigned short int affichage_digit_unite = 0;

unsigned short int afficheur_actif = 3;
         // 3 = afficheur des dizaines (broche RA3)
         // 4 = afficheur des units (broche RA4)
unsigned short int valeur_a_afficher ;
unsigned short int etat_courant = 0;

unsigned short int transition_bp_reset = 0;
unsigned short int transition_bp_reprise = 0;
unsigned short int transition_bp_start_pause = 0;

unsigned short int etatanterieur_bp_reset = 1;         // BP ouvert lors de la derniere scrutation
unsigned short int etatanterieur_bp_reprise = 1;
unsigned short int etatanterieur_bp_start_pause = 1;
unsigned short int etat_port ;                  // variable temporaire

unsigned short int compteur_00 = 0;      // vaut 1 quand le compteur passe  00
unsigned short int compteur_15 = 0;      // vaut 1 quand le compteur passe  15
unsigned short int compteur_14 = 0;      // vaut 1 quand le compteur passe  14
unsigned short int compteur_13 = 0;      // vaut 1 quand le compteur passe  13
unsigned short int compteur_12 = 0;      // vaut 1 quand le compteur passe  12
unsigned short int compteur_11 = 0;      // vaut 1 quand le compteur passe  11
unsigned short int compteur_10 = 0;      // vaut 1 quand le compteur passe  10

unsigned short int compteur_fige = 1;    // le compteur est fig
unsigned short int compteur_buzzer_fin = 3 ;     // 3 secondes

void interrupt()
{
     // routine d'interruption
     // 1 seule source d'interruption : TMR0 en mode timer
     // toutes les 256*2 = 512 cycles = 102,4 s (prescaler 1:2) quartz 20 MHz
     // toutes les 256*1 cycles = 102,4 s (prescaler 1:1) quartz 10 MHz

     if ((INTCON.T0IE == 1) && (INTCON.T0IF == 1))
     {
        rafraichissement() ;
		    INTCON.T0IF = 0; //on efface le drapeau du timer TMR0
     }
}

void main()
{
     // initialisation

     PORTA = 0b00000000;  	// mise  0 des sorties du port A
     PORTB = 0b00000000;  	// mise  0 des sorties du port B

     CMCON	= 0b00000111;		// inactivation des comparateurs analogiques

     OPTION_REG = 0b00000000;
     /*
	   bit 7 (/RBPU) = 0 : activation des rsistances de pull-up du port B
	   bit 6 (INTEDG)= 0 : (non utilis)
	   bit 5 (T0CS) = 0 : TMR0 Clock Source Select = OSC2/CLKOUT (mode timer)
	   bit 4 (T0SE) = 0 : (non utilis)
	   bit 3 (PSA) = 1
	   bit 2 (PS2)= 0
	   bit 1 (PS1) = 0
	   bit 0 (PS0) = 0
	   Avec un quartz 10 MHz : prescaler 1:1	(PSA PS2 PS1 PS0) = (1000)
	   Avec un quartz 20 MHz : prescaler 1:2	(PSA PS2 PS1 PS0) = (0000)
     */

     TRISA = 0b00000111;
	   // bit 0 du port A (RA0) = 1 : configuration en entre (broche non utilise)
	   // bit 1 du port A (RA1) = 1 : configuration en entre (broche non utilise)
	   // bit 2 du port A (RA2) = 1 : configuration en entre (broche non utilise)
	   // bit 3 du port A (RA3) = 0 : configuration en sortie (afficheur des dizaines)
	   // bit 4 du port A (RA4) = 0 : configuration en sortie (afficheur des units)

     TRISB = 0b10000011;
	   // bit 0 du port B (RB0) = 1 : configuration en entre (bouton poussoir REPRISE)
	   // bit 1 du port B (RB1) = 1 : configuration en entre (bouton poussoir RESET)
	   // bit 2 du port B (RB2) = 0 : configuration en sortie (entre A du 7447)
	   // bit 3 du port B (RB3) = 0 : configuration en sortie (entre B du 7447)
	   // bit 4 du port B (RB4) = 0 : configuration en sortie (entre C du 7447)
	   // bit 5 du port B (RB5) = 0 : configuration en sortie (entre D du 7447)
	   // bit 6 du port B (RB6) = 0 : configuration en sortie (commande du Buzzer)
	   // bit 7 du port B (RB7) = 1 : configuration en entre (bouton poussoir START/PAUSE)

	  PORTB.BIT_BUZZER = 0;	// buzzer off

    TMR0 = 0x00;     // mise  zero du timer0

	  INTCON = 0b10100000;
		 // bit 7 (GIE) = 1 : autorisation globale des interruptions
		 // bit 5 (T0IE) = 1 : autorisation de l'interruption TMR0
		 // bit 2 (T0IF)= 0 : on efface le drapeau de l'interruption TMR0
     // les autres bits sont inutiliss (valeur par dfaut = 0)

	do
	{
		// on attend le dbordement de TMR0 (0xFF -> 0x00)
		// ce qui gnre une interruption
	} while(1);
}

void rafraichissement()
{
     compteur_tmr0--;
     if (compteur_tmr0 == 0)
     {
          // on arrive ici toutes les 1/257 seconde (environ)
          // 38 * 102,4 s = 3,8912 ms
          // 1/257 = 3,891051 ms
          compteur_tmr0 = COMPTEUR_TMR0_INIT; // rinitialisation
          
          ///////////////

          compteur_bp--;
          if (compteur_bp == 0)
          {
             // on arrive ici toutes les 40*3,8912 ms # 156 ms
             compteur_bp = COMPTEUR_BP_INIT; // rinitialisation
             //////////////////
             // Scrutation des 3 boutons poussoirs
             transition_bp_reset = test_bp_reset();
             transition_bp_reprise = test_bp_reprise();
             transition_bp_start_pause = test_bp_start_pause();
          }
          
          /////////////////////
          
          if (compteur_fige == 0)
          {
          compteur--;
          if (compteur == 0)
          {
              // on arrive ici toute les 1 seconde (environ) si le dcomptage est actif
              // 3,8912 ms * 257 = 1,0000384 s
              compteur = COMPTEUR_INIT;  // rinitialisation
          
              /// bip de fin (dure 3 secondes)
              if ((affichage_digit_dizaine == 0) && (affichage_digit_unite == 1))
              {
                    // passage 01  00
                    compteur_00 = 1;
              }
              
              // 1er bip de rappel (dure 1 seconde)
              if ((affichage_digit_dizaine == 1) && (affichage_digit_unite == 6))
              {
                    // passage 16  15
                    compteur_15 = 1;
              }
              if ((affichage_digit_dizaine == 1) && (affichage_digit_unite == 5))
              {
                    // passage 15  14 // fin du bip
                    compteur_14 = 1;
              }
              // 2eme bip de rappel (dure 1 seconde)
              if ((affichage_digit_dizaine == 1)&& (affichage_digit_unite == 4))
              {
                    // passage 14  13
                    compteur_13 = 1;
              }
              if ((affichage_digit_dizaine == 1) && (affichage_digit_unite == 3))
              {
                    // passage 13  12   // fin du bip
                    compteur_12 = 1;
              }
              // 3eme bip de rappel (dure 1 seconde)
              if ((affichage_digit_dizaine == 1)&& (affichage_digit_unite == 2))
              {
                    // passage 12  11
                    compteur_11 = 1;
              }
              if ((affichage_digit_dizaine == 1) && (affichage_digit_unite == 1))
              {
                    // passage 11  10   // fin du bip
                    compteur_10 = 1;
              }
              ////////////////////
              

                  if (affichage_digit_unite == 0)
                  {
                     if (affichage_digit_dizaine == 0)
                     {
                          //  affichage 00
                          if (etat_courant == 3) compteur_buzzer_fin--;
                     }
                     else
                     {
                        affichage_digit_unite = 9;
                        affichage_digit_dizaine--;
                     }
                  }
                  else
                  {
                       affichage_digit_unite--;
                  }

          }
          }
          ///////////////////////////////////////////////////////////
          // rafraichissement des afficheurs (toutes les 1/257 seconde)
          
          if (afficheur_actif == 3) afficheur_actif = 4;
          else afficheur_actif = 3;
          // 3 = afficheur des dizaines (broche RA3)
          // 4 = afficheur des units (broche RA4)
          // squence 3 -> 4 -> 3 -> 4 ->  etc (multiplexage)
          //////////////////////////////////////////
          
          if (afficheur_actif == 3)   // digit des dizaines
          {
              valeur_a_afficher = affichage_digit_dizaine;
              PORTA.BIT_ANODE_DIGIT_DIZAINE = 0;    // digit des dizaines actif
              PORTA.BIT_ANODE_DIGIT_UNITE = 1;      // digit des units teint
              
          }
          if (afficheur_actif == 4)   // digit des units
          {
              valeur_a_afficher = affichage_digit_unite;
              PORTA.BIT_ANODE_DIGIT_DIZAINE = 1;    // digit des dizaines teint
              PORTA.BIT_ANODE_DIGIT_UNITE = 0;      // digit des units actif
          }

          switch (valeur_a_afficher)  {
          
                 case 0 :  // on affiche 0 sur l'afficheur actif
                 PORTB.BIT_7447_A = 0;
                 PORTB.BIT_7447_B = 0;
                 PORTB.BIT_7447_C = 0;
                 PORTB.BIT_7447_D = 0;
                 break;
          
                 case 1 :  // on affiche 1 sur l'afficheur actif
                 PORTB.BIT_7447_A = 1;
                 PORTB.BIT_7447_B = 0;
                 PORTB.BIT_7447_C = 0;
                 PORTB.BIT_7447_D = 0;
                 break;
          
                 case 2 :  // on affiche 2 sur l'afficheur actif
                 PORTB.BIT_7447_A = 0;
                 PORTB.BIT_7447_B = 1;
                 PORTB.BIT_7447_C = 0;
                 PORTB.BIT_7447_D = 0;
                 break;
          
                 case 3 :  // on affiche 3 sur l'afficheur actif
                 PORTB.BIT_7447_A = 1;
                 PORTB.BIT_7447_B = 1;
                 PORTB.BIT_7447_C = 0;
                 PORTB.BIT_7447_D = 0;
                 break;
                 
                 case 4 :  // on affiche 4 sur l'afficheur actif
                 PORTB.BIT_7447_A = 0;
                 PORTB.BIT_7447_B = 0;
                 PORTB.BIT_7447_C = 1;
                 PORTB.BIT_7447_D = 0;
                 break;
                 
                 case 5 :  // on affiche 5 sur l'afficheur actif
                 PORTB.BIT_7447_A = 1;
                 PORTB.BIT_7447_B = 0;
                 PORTB.BIT_7447_C = 1;
                 PORTB.BIT_7447_D = 0;
                 break;
                 
                 case 6 :  // on affiche 6 sur l'afficheur actif
                 PORTB.BIT_7447_A = 0;
                 PORTB.BIT_7447_B = 1;
                 PORTB.BIT_7447_C = 1;
                 PORTB.BIT_7447_D = 0;
                 break;
                 
                 case 7 :  // on affiche 7 sur l'afficheur actif
                 PORTB.BIT_7447_A = 1;
                 PORTB.BIT_7447_B = 1;
                 PORTB.BIT_7447_C = 1;
                 PORTB.BIT_7447_D = 0;
                 break;
                 
                 case 8 :  // on affiche 8 sur l'afficheur actif
                 PORTB.BIT_7447_A = 0;
                 PORTB.BIT_7447_B = 0;
                 PORTB.BIT_7447_C = 0;
                 PORTB.BIT_7447_D = 1;
                 break;
                 
                 case 9 :  // on affiche  sur l'afficheur actif
                 PORTB.BIT_7447_A = 1;
                 PORTB.BIT_7447_B = 0;
                 PORTB.BIT_7447_C = 0;
                 PORTB.BIT_7447_D = 1;
                 break;
          }
          
          ///////////////////////////////////////////////
          ////////////////////  Test changement d'tat
          // on arrive ici toutes les 1/257 seconde (environ)

              switch (etat_courant)  {

              case 0 :  // tat initial : compteur fig  60
              PORTB.BIT_BUZZER = 0;	// buzzer off
              compteur_fige = 1; // compteur fig
              compteur_15 = 0;
              compteur_13 = 0;
              compteur_11 = 0;
              compteur_14 = 0;
              compteur_12 = 0;
              compteur_10 = 0;
              compteur_00 = 0;
              compteur = COMPTEUR_INIT;
              compteur_tmr0 = COMPTEUR_TMR0_INIT;
              compteur_buzzer_fin = 3 ; //  3 secondes
              affichage_digit_unite = 0;
              affichage_digit_dizaine = 6;

              if (transition_bp_start_pause == 1)
              {
                 transition_bp_start_pause = 0;
                 etat_courant = 1;
              }
              break;

              case 1 :  // phase normale : dcomptage
              PORTB.BIT_BUZZER = 0;	// buzzer off
              compteur_fige = 0;
              
              if (compteur_15 == 1)
              {
                 compteur_15 = 0;
                 etat_courant = 4;
                 break;
              }
              if (compteur_13 == 1)
              {
                 compteur_13 = 0;
                 etat_courant = 5;
                 break;
              }
              if (compteur_11 == 1)
              {
                 compteur_11 = 0;
                 etat_courant = 6;
                 break;
              }
              if (compteur_00 == 1)
              {
                 compteur_00 = 0;
                 etat_courant = 3;
                 break;
              }
              if (transition_bp_start_pause == 1)
              {
                 transition_bp_start_pause = 0;
                 etat_courant = 2;
                 break;
              }
              if (transition_bp_reset == 1)
              {
                 transition_bp_reset = 0;
                 etat_courant = 0;
                 break;
              }
              break;

              case 2 :  // pause : compteur fig
              PORTB.BIT_BUZZER = 0;	// buzzer off
              compteur_fige = 1; // compteur fig
              if (transition_bp_reprise == 1)
              {
                 transition_bp_reprise = 0;
                 etat_courant = 1;
                 break;
              }
              if (transition_bp_reset == 1)
              {
                 transition_bp_reset = 0;
                 etat_courant = 0;
                 break;
              }
              break;

              case 3 :  //  fin du comptage (00)
              PORTB.BIT_BUZZER = 1;	// buzzer on
              compteur_fige = 0; // attention : compteur non fig (mais on affiche 00)
              compteur_00 = 0;
              
              if (compteur_buzzer_fin == 0)
              {
                 etat_courant = 0;  // fin de la tempo de 3 secondes
                 break;
              }
              if (transition_bp_reset == 1)
              {
                 transition_bp_reset = 0;
                 etat_courant = 0;
                 break;
              }
              break;

              case 4 :  // 1er bip de rappel
              PORTB.BIT_BUZZER = 1;	// buzzer on
              compteur_fige = 0;
              compteur_15 = 0;
              if (compteur_14 == 1)
              {
                  compteur_14 = 0;
                  etat_courant = 1;
                  PORTB.BIT_BUZZER = 0;	// buzzer off
                  break;
              }
              if (transition_bp_start_pause == 1)
              {
                 transition_bp_start_pause = 0;
                 etat_courant = 2;
                 PORTB.BIT_BUZZER = 0;	// buzzer off
                 break;
              }
              if (transition_bp_reset == 1)
              {
                 transition_bp_reset = 0;
                 etat_courant = 0;
                 PORTB.BIT_BUZZER = 0;	// buzzer off
                 break;
              }

              break;
              
              case 5 :  // 2eme bip de rappel
              PORTB.BIT_BUZZER = 1;	// buzzer on
              compteur_fige = 0;
              compteur_13 = 0;
              if (compteur_12 == 1)
              {
                  compteur_12 = 0;
                  etat_courant = 1;
                  PORTB.BIT_BUZZER = 0;	// buzzer off
                  break;
              }
              if (transition_bp_start_pause == 1)
              {
                 transition_bp_start_pause = 0;
                 etat_courant = 2;
                 PORTB.BIT_BUZZER = 0;	// buzzer off
                 break;
              }
              if (transition_bp_reset == 1)
              {
                 transition_bp_reset = 0;
                 etat_courant = 0;
                 PORTB.BIT_BUZZER = 0;	// buzzer off
                 break;
              }
              break;
              
              case 6 :  // 3eme bip de rappel
              PORTB.BIT_BUZZER = 1;	// buzzer on
              compteur_fige = 0;
              compteur_11 = 0;
              if (compteur_10 == 1)
              {
                  compteur_10 = 0;
                  etat_courant = 1;
                  PORTB.BIT_BUZZER = 0;	// buzzer off
                  break;
              }
              if (transition_bp_start_pause == 1)
              {
                 transition_bp_start_pause = 0;
                 etat_courant = 2;
                 PORTB.BIT_BUZZER = 0;	// buzzer off
                 break;
              }
              if (transition_bp_reset == 1)
              {
                 transition_bp_reset = 0;
                 etat_courant = 0;
                 PORTB.BIT_BUZZER = 0;	// buzzer off
                 break;
              }
              break;
              
              }
     }
     return;
}

unsigned short int test_bp_reset()
{
     // test du bouton poussoir RESET
     // etatanterieur_bp_reset = 1;         // BP ouvert lors de la derniere scrutation
     // etatanterieur_bp_reset = 0;         // BP ferm lors de la derniere scrutation
     // on dtecte le passage de 1  0

     etat_port = PORTB.BIT_BP_RESET;   // lecture du niveau du port

     if ((etat_port == 0) && (etatanterieur_bp_reset == 1))
     {
           etatanterieur_bp_reset = etat_port;
           return 1; // BP a t appuy
     }
     else
     {
           etatanterieur_bp_reset = etat_port;
           return 0; // BP inactif
     }
}

unsigned short int test_bp_reprise()
{
     // test du bouton poussoir REPRISE

     etat_port = PORTB.BIT_BP_REPRISE;   // lecture du niveau du port

     if ((etat_port == 0) && (etatanterieur_bp_reprise == 1))
     {
           etatanterieur_bp_reprise = etat_port;
           return 1; // BP a t appuy
     }
     else
     {
           etatanterieur_bp_reprise = etat_port;
           return 0; // BP inactif
     }
}

unsigned short int test_bp_start_pause()
{
     // test du bouton poussoir START_PAUSE
     etat_port = PORTB.BIT_BP_START_PAUSE;   // lecture du niveau du port

     if ((etat_port == 0) && (etatanterieur_bp_start_pause == 1))
     {
           etatanterieur_bp_start_pause = etat_port;
           return 1; // BP a t appuy
     }
     else
     {
           etatanterieur_bp_start_pause = etat_port;
           return 0; // BP inactif
     }
}

